﻿using System;
using System.Collections.Generic;
using AppOptics.Instrumentation;
using Microsoft.AspNetCore;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;

namespace coretestapp
{

    public class Startup
    {
        string spanName = "example-trace";

        public void StartTrace()
        {
            System.Console.WriteLine("Starting trace");
            StartTraceEvent startEvent = Trace.StartTrace(spanName);
            startEvent.AddInfo("Key1", "extra info on trace entry");
            startEvent.Report();
        }

        public void EndTrace()
        {
            System.Console.WriteLine("Ending trace");
            EndTraceEvent endEvent = Trace.EndTrace(spanName);
            endEvent.AddInfo("Key2", "extra info on trace exit");
            endEvent.Report();
        }

        public void TracedMethod()
        {
            System.Console.WriteLine("\n****** Enter TracedMethod ***** \n");

            System.Console.WriteLine("Starting span for TracedMethod");

            // Note: the name must match for span start and end events. It
            // should also be the same for info events within this span, in
            // order for the info events to be displayed in the Raw Data view.
            string spanName = "example-span";

            EntryTraceEvent eventEntry = Trace.CreateEntryEvent(spanName);
            eventEntry.AddInfo("Key1", "extra info on span entry");
            eventEntry.Report();

            // Generate some info events
            for(int i = 0; i < 5; i++){
                InfoTraceEvent e = Trace.CreateInfoEvent(spanName);
                e.AddInfo("Key", "extra info " + i);
                e.Report();
            }

            System.Console.WriteLine("Ending span");
            ExitTraceEvent eventExit = Trace.CreateExitEvent(spanName);
            eventExit.AddInfo("Key2", "extra info on span exit");
            eventExit.Report();

            System.Console.WriteLine("\n****** Exit TracedMethod ***** \n");
        }

        public void Configure(IApplicationBuilder app)
        {
            // Add the AppOptics.Instrumentation Middleware
            // Note: this needs to intercept the request before it is
            // processed by the first MVC middleware, so should be added
            // before the first MVC middleware in Startup.
            app.UseAppopticsApm();

            app.Run(async context =>
                    {
                        // Using the AppOpticsApm Middleware will automatically start and end traces
                        // and report metrics.  Alternatively, the reporting of StartTraceEvent and
                        // EndTraceEvents can be used as shown in the StartTrace and EndTrace methods.
                        // As the AppOpticsApm Middleware is being used in this example those methods have
                        // been commented out.

                        // StartTrace();
                        TracedMethod();
                        await context.Response.WriteAsync("Hello, World!");
                        // EndTrace();
                    });
        }
    }

    public class Program
    {
        public static void Main(string[] args)
        {
            System.Console.WriteLine("Running application ...");

            CreateWebHostBuilder(args).Build().Run();

            System.Console.WriteLine("Application has stopped.");
        }

        public static IWebHostBuilder CreateWebHostBuilder(string[] args) =>
            WebHost.CreateDefaultBuilder(args)
            .UseStartup<Startup>();
    }
}
